package dev.shadowsoffire.apotheosis.adventure.affix;

import net.minecraft.util.Mth;
import net.minecraft.world.level.storage.loot.LootContext;
import it.unimi.dsi.fastutil.objects.ObjectArrayList;
import net.minecraft.world.item.enchantment.Enchantment;
import java.util.Map;
import net.minecraft.world.phys.HitResult;
import net.minecraft.server.level.ServerPlayer;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraft.core.BlockPos;
import net.minecraft.world.level.LevelAccessor;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.InteractionResult;
import net.minecraft.world.item.context.UseOnContext;
import net.minecraft.world.entity.projectile.AbstractArrow;
import javax.annotation.Nullable;
import net.minecraft.world.entity.Entity;
import net.minecraft.world.entity.LivingEntity;
import net.minecraft.world.entity.MobType;
import net.minecraft.world.damagesource.DamageSource;
import net.minecraft.network.chat.Component;
import net.minecraft.network.chat.MutableComponent;
import net.minecraft.world.entity.ai.attributes.AttributeModifier;
import net.minecraft.world.entity.ai.attributes.Attribute;
import java.util.function.BiConsumer;
import net.minecraft.world.entity.EquipmentSlot;
import dev.shadowsoffire.apotheosis.adventure.loot.LootRarity;
import net.minecraft.world.item.ItemStack;
import dev.shadowsoffire.placebo.reload.DynamicHolder;

record AffixInstance(DynamicHolder<? extends Affix> affix, ItemStack stack, DynamicHolder<LootRarity> rarity, float level) {
    public boolean isValid() {
        return this.affix.isBound() && this.rarity.isBound();
    }
    
    private Affix afx() {
        return (Affix)this.affix.get();
    }
    
    private LootRarity rty() {
        return (LootRarity)this.rarity.get();
    }
    
    public void addModifiers(final EquipmentSlot type, final BiConsumer<Attribute, AttributeModifier> map) {
        this.afx().addModifiers(this.stack, this.rty(), this.level, type, map);
    }
    
    public MutableComponent getDescription() {
        return this.afx().getDescription(this.stack, this.rty(), this.level);
    }
    
    public Component getAugmentingText() {
        return this.afx().getAugmentingText(this.stack, this.rty(), this.level);
    }
    
    public Component getName(final boolean prefix) {
        return this.afx().getName(prefix);
    }
    
    public int getDamageProtection(final DamageSource source) {
        return this.afx().getDamageProtection(this.stack, this.rty(), this.level, source);
    }
    
    public float getDamageBonus(final MobType creatureType) {
        return this.afx().getDamageBonus(this.stack, this.rty(), this.level, creatureType);
    }
    
    public void doPostAttack(final LivingEntity user, @Nullable final Entity target) {
        this.afx().doPostAttack(this.stack, this.rty(), this.level, user, target);
    }
    
    public void doPostHurt(final LivingEntity user, @Nullable final Entity attacker) {
        this.afx().doPostHurt(this.stack, this.rty(), this.level, user, attacker);
    }
    
    public void onArrowFired(final LivingEntity user, final AbstractArrow arrow) {
        this.afx().onArrowFired(this.stack, this.rty(), this.level, user, arrow);
    }
    
    @Nullable
    public InteractionResult onItemUse(final UseOnContext ctx) {
        return this.afx().onItemUse(this.stack, this.rty(), this.level, ctx);
    }
    
    public float onShieldBlock(final LivingEntity entity, final DamageSource source, final float amount) {
        return this.afx().onShieldBlock(this.stack, this.rty(), this.level, entity, source, amount);
    }
    
    public void onBlockBreak(final Player player, final LevelAccessor world, final BlockPos pos, final BlockState state) {
        this.afx().onBlockBreak(this.stack, this.rty(), this.level, player, world, pos, state);
    }
    
    public float getDurabilityBonusPercentage(@Nullable final ServerPlayer user) {
        return this.afx().getDurabilityBonusPercentage(this.stack, this.rty(), this.level, user);
    }
    
    public void onArrowImpact(final AbstractArrow arrow, final HitResult res, final HitResult.Type type) {
        this.afx().onArrowImpact(arrow, this.rty(), this.level, res, type);
    }
    
    public boolean enablesTelepathy() {
        return this.afx().enablesTelepathy();
    }
    
    public float onHurt(final DamageSource src, final LivingEntity ent, final float amount) {
        return this.afx().onHurt(this.stack, this.rty(), this.level, src, ent, amount);
    }
    
    public void getEnchantmentLevels(final Map<Enchantment, Integer> enchantments) {
        this.afx().getEnchantmentLevels(this.stack, this.rty(), this.level, enchantments);
    }
    
    public void modifyLoot(final ObjectArrayList<ItemStack> loot, final LootContext ctx) {
        this.afx().modifyLoot(this.stack, this.rty(), this.level, loot, ctx);
    }
    
    public AffixInstance withNewLevel(final float level) {
        return new AffixInstance(this.affix, this.stack, this.rarity, Mth.m_14036_(level, 0.0f, 1.0f));
    }
}
